/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>

// define some constants to make referencing in/outs clearer


// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *in1  = buffer_offset + input1_buffer;
    float *in2  = buffer_offset + input2_buffer;
    float *in3  = buffer_offset + input3_buffer;
    float *in4  = buffer_offset + input4_buffer;
    float *in5  = buffer_offset + input5_buffer;
    float *out1 = buffer_offset + output1_buffer;

    while(--sampleFrames >= 0) // sampleFrames = how many samples to process (can vary). repeat (loop) that many times
    {
		// do the actual processing 
		
		if (*in1 > *in3) mem = true;
		if (*in2 > *in3) mem = false;
		if ((mybool == true ) && (*in1 > *in3 )) mem = true;
         
		if ( mem == true )*out1 = *in4; else 
			              *out1 = *in5;
		// increment the pointers (move to next sample in buffers)
		in1++;
		in2++;
		in3++;
        in4++;
		in5++;
		out1++;
    }
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	SEModule_base::open();
	mem = false;

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);
}

Module::~Module()
{
	// This is where you free any memory/resources your module has created
}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "FlipFlop2";								// !!TODO!!

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "ralphs FF2";						// !!TODO!!

	// Info, may include Author, Web page whatever


#if defined(MINGW) || defined(__GNUC__)
    properties->about =
		"by ralph a. cronin \n  FAKE MUSIC \n ( searching jazz bass to my piano 2009 ) \n ************* \n -----enjoy Synthedit------\n ************* \n by Jeff Mc Clintock" ;
#elif defined _MSC_VER
	properties->about =
		"*********************************************\n*                       FLIPFLOP2                          *\n*                set this to output                       *\n*           reset that to output                        *\n*     care about switch_level                   *\n___________________________________\n             by ralph a. cronin \n  >>>>>  FAKE MUSIC  <<<<< \n www.ralphcronin.webs.com \n( searching jazz bass to my piano 2009 ) \n ******************************************** \n        -----enjoy Synthedit------\n ******************************************** \n by Jeff Mc Clintock" ;
#else
	properties->about =
		"by ralph a. cronin \n  FAKE MUSIC \n ( searching jazz bass to my piano 2009 ) \n ************* \n -----enjoy Synthedit------\n ************* \n by Jeff Mc Clintock" ;


#endif
	return true;

}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case 0:
		properties->name				= "set";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case 1:
		properties->name				= "reset";
		properties->variable_address	= &input2_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
    case 2:
		properties->name				= "switch_level";
		properties->variable_address	= &input3_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
        properties->datatype_extra      = "10,-10,10,-10";
        properties->default_value		= "0";
		break;
    case 3:
		properties->name				= "this";
		properties->variable_address	= &input4_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
        properties->datatype_extra      = "10,-10,10,-10";
        properties->default_value		= "10";
		break;
    case 4:
		properties->name				= "that";
		properties->variable_address	= &input5_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->datatype_extra      = "10,-10,10,-10";
        properties->default_value		= "-10";
		break;

		// typical output plug
	case 5:
		properties->name				= "SET-PRIORITY";
		properties->variable_address	= &mybool;
		properties->direction			= DR_PARAMETER;
		properties->datatype			= DT_BOOL;
        properties->default_value		= "1";
		break;
	case 6:
		properties->name				= "Q";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
   
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
	
	// query the 'state of the input plugs...
	//     ST_RUN     = Normal Streaming Audio        (e.g. from an oscillator)
	//     ST_STATIC  = Fixed, unchanging input value (e.g. a slider at rest)
	
	state_type in_stat;
	state_type out_stat = ST_STATIC;
	for (int i = 0; i <= 5; i++)
	{
	 in_stat = getPin(i)->getStatus();
	 if( in_stat > ST_STATIC ) 
	         {
				 out_stat = ST_RUN; 
				 break;
	         }
	}
	// 'transmit' new output status to next module 'downstream'
	getPin(PN_OUTPUT1)->TransmitStatusChange( SampleClock(), out_stat);// statt out_stat ralph
}

